import {Component, EventEmitter, Output, ViewChild} from '@angular/core';
import {MatExpansionModule, MatExpansionPanel} from "@angular/material/expansion";
import {MatChipListboxChange, MatChipsModule} from '@angular/material/chips';
import {FormControl, FormsModule, ReactiveFormsModule} from "@angular/forms";
import {catchError} from "rxjs";
import {LoadingIndicatorComponent} from "@component/common/loading-indicator/loading-indicator.component";
import {GroupResponse} from "@api/v1/groupResponse";
import {FacultyResponse} from "@api/v1/facultyResponse";
import {FacultyService} from "@api/v1/faculty.service";
import {GroupService} from "@api/v1/group.service";

@Component({
  selector: 'app-group',
  standalone: true,
  imports: [
    MatExpansionModule,
    MatChipsModule,
    ReactiveFormsModule,
    LoadingIndicatorComponent,
    FormsModule
  ],
  templateUrl: './group.component.html',
  styleUrl: './group.component.css',
  providers: [FacultyService, GroupService]
})

export class GroupComponent {
  protected facultyId: number | null = null;
  protected courseNumber: number | null = null;

  protected filteredGroups: GroupResponse[] = [];
  protected courseNumbers: number[] = [];
  private groups: GroupResponse[] = [];

  protected formChipCourse: FormControl = new FormControl();
  protected formChipGroup: FormControl = new FormControl();

  protected faculties: FacultyResponse[] = [];

  @ViewChild('courseNumberPanel') courseNumberPanel!: MatExpansionPanel;
  @ViewChild('groupPanel') groupPanel!: MatExpansionPanel;

  protected facultiesLoaded: boolean | null = false;
  protected groupsLoaded: boolean | null = false;

  @Output() eventResult = new EventEmitter<number>();

  constructor(private facultyApi: FacultyService, private groupApi: GroupService) {
    this.loadFaculties();
  }

  protected loadFaculties() {
    this.facultiesLoaded = false;
    this.facultyApi.getFaculties()
      .pipe(catchError(error => {
        this.facultiesLoaded = null;
        throw error;
      }))
      .subscribe(data => {
        this.faculties = data;
        this.facultiesLoaded = true;
      });
  }

  private filteringCourseNumber() {
    this.courseNumbers = Array.from(
      new Set(
        this.groups
          .filter(x => x.facultyId === this.facultyId)
          .map(x => x.courseNumber)
      )
    ).sort((a, b) => a - b);
  }

  private filteringGroup() {
    this.filteredGroups = this.groups.filter(x => x.facultyId === this.facultyId && x.courseNumber === this.courseNumber);
  }

  protected loadCourseGroup() {
    if (this.groups.length === 0) {
      this.groupsLoaded = false;

      this.groupApi.getGroups().pipe(
        catchError(error => {
          this.groupsLoaded = null;
          throw error;
        })
      ).subscribe(data => {
        this.groups = data;
        if (this.courseNumber === null)
          this.filteringCourseNumber();
        else
          this.filteringGroup();

        this.groupsLoaded = true;
      });

      return
    }

    if (this.courseNumber === null)
      this.filteringCourseNumber();
    else
      this.filteringGroup();
  }

  protected chooseFaculty(event: MatChipListboxChange) {
    this.courseNumber = null;
    this.groups = [];
    this.formChipGroup.reset();
    this.formChipCourse.reset();

    if (event.value === undefined || event.value === null) {
      this.facultyId = null;
      return;
    }
    this.facultyId = event.value;
    this.courseNumberPanel.open();

    this.loadCourseGroup();
  }

  protected chooseCourseNumber(event: MatChipListboxChange) {
    this.filteredGroups = [];
    this.formChipGroup.reset();

    if (event.value === undefined || event.value === null) {
      this.courseNumber = null;
      return;
    }

    this.courseNumber = event.value;
    this.groupPanel.open();
    this.loadCourseGroup();
  }

  protected chooseGroup(event: MatChipListboxChange) {
    if (event.value === undefined || event.value === null)
      return;

    this.groupPanel.close();
    this.eventResult.emit(event.value);
  }
}
